package com.stormtales.notevault.network.sync;

import android.content.Context;
import com.stormtales.notevault.data.entities.Song;
import com.stormtales.notevault.data.repositories.SongRepository;
import com.stormtales.notevault.data.repositories.SongSyncRepository;
import com.stormtales.notevault.network.sync.models.BatchCreateResponse;
import com.stormtales.notevault.network.sync.models.BatchModifyResponse;
import com.stormtales.notevault.network.sync.models.SongModifyBatchResponse;
import com.stormtales.notevault.network.sync.models.UploadResponse;
import com.stormtales.notevault.ui.gallery.GalleryViewModel;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

public class SongSyncModule {
    private SongRepository songRepository;
    private SongSyncRepository songSyncRepository;
    private SongSyncService songSyncService;
    private GalleryViewModel syncViewModel;

    public SongSyncModule(Context context, GalleryViewModel syncViewModel) {
        this.songRepository = new SongRepository(context);
        this.songSyncRepository = new SongSyncRepository(context);
        this.songSyncService = new SongSyncService(context);
        this.syncViewModel = syncViewModel;
    }

    public void syncCreatedSongs() {
        songSyncRepository.loadCreatedSongs(result -> {
            songSyncService.syncCreatedSongs(result, response -> {
                songSyncRepository.markCreatedSongsAsSynced(response);
                if(response.getCreateResponses().isEmpty()) {
                    syncViewModel.finishCreateSongSyncing();
                } else {
                    uploadCreatedNoteSheets(result, response);
                }
            });
        });
    }

    public void uploadCreatedNoteSheets(List<Song> result, BatchCreateResponse response) {
        List<Integer> songIDs = result.stream().map(Song::getLocalID).collect(Collectors.toList());
        songSyncRepository.getNoteSheetFilesBySongIDs(noteSheets -> {
            songSyncService.uploadNoteSheetsOfCreatedSongs(noteSheets, response, uploadResponses -> {
                songSyncRepository.markCreatedNoteSheetsAsSynced(uploadResponses);
                syncViewModel.finishCreateSongSyncing();
            });
        });
    }

    public void syncModifiedSongs() {
        songSyncRepository.loadModifiedSongs(result -> {
            songSyncService.syncModifiedSongs(result, (FinishSongSyncingCallback<SongModifyBatchResponse>) response -> {
                songSyncRepository.markModifiedSongsAsSynced(response);
                uploadModifiedNoteSheets();
            });
        });
    }

    public void uploadModifiedNoteSheets() {
        songSyncRepository.loadModifiedNoteSheets(modifiedNoteSheets -> {
            songSyncService.uploadModifiedNoteSheets(modifiedNoteSheets, new SongSyncService.UploadNoteSheetCallback() {
                @Override
                public void finishUploadNoteSheets(List<UploadResponse> uploadResponses) {
                    songSyncRepository.markModifiedNoteSheetsAsSynced(uploadResponses);
                    syncViewModel.finishModifiedSongSyncinc();
                }
            });
        });
    }

    public void syncDeletedSongs() {
        songSyncRepository.loadDeletedSongs(result -> {
            songSyncService.syncDeletedSong(result, (remoteDeletedSongs, localDeletedSongs) -> {
                songSyncRepository.markDeletedSongsAsSynced(remoteDeletedSongs, localDeletedSongs);
                syncViewModel.finishDeleteSongSyncinc();
            });
        });
    }

    public interface FinishSongCreateSyncingCallback {
        void finishSongSyncing(BatchCreateResponse response);
    }

    public interface FinishSongSyncingCallback<T> {
        void finishSongSyncing(T response);
    }
}
